require 'rubygems'
require 'json'
require 'pp'
require 'open-uri'
require 'net/http'
require 'rest'
require 'cocoapods-core'
require 'cocoapods-core/specification/json'
require 'cocoapods'
require 'cocoapods/sandbox/path_list'
require 'cocoapods/project'
require 'fileutils'


module BDAwesomeTransmit
  class BDPodsTransLog

    attr_accessor :repoName

    attr_accessor :repo

    def initialize(repoName,dev_path = nil)
      @repoName = repoName
      
      @dev_path = dev_path
    end


    def run
      getComponentVersion
      requirePodspecInfo
      requirePodspecInfoIfNeeded
      establishGitRepo
      trans_dev_if_needed
    end

    private

    def trans_dev_if_needed
       return unless @dev_path
       @@project = Xcodeproj::Project.open('Pods/Pods.xcodeproj')

       unless isContainDevPod
        dev_pods = @@project.main_group.new_group('Development Pods') 
        @@project.main_group.children.pop()
        @@project.main_group.children.insert(1,dev_pods)
       end
       pod_g = pod_group(@repoName)
       unless isInDevPod(@repoName)
         pod_g.move(development_pod)
       end
       `
       test -d "#{@dev_path}/#{@repoName}" || mv Pods/"#{@repoName}" "#{@dev_path}"
       rm -rf Pods/"#{@repoName}"
       last=\`pwd\`
       cd "#{@dev_path}"
       ln -fs \`pwd\`/"#{@repoName}" $last/Pods
       `
       if pod_g.is_a? Xcodeproj::Project::Object::PBXFileReference
          
          pod_proj = Xcodeproj::Project.open(pod_g.real_path)
          pod_proj_g = pod_proj.groups.find { |group| group.name == @repoName }
          pod_proj_g.set_path("#{@dev_path}/#{@repoName}")
          pod_proj_g.groups.find{|group| group.name == 'Support Files'}.set_path("#{Dir.pwd}/Pods/Target Support Files/#{@repoName}")
          pod_proj.save
          @@project.save
       else
          pod_g.set_path("#{@dev_path}/#{@repoName}")
          pod_g.groups.find{|group| group.name == 'Support Files'}.set_path("#{Dir.pwd}/Pods/Target Support Files/#{@repoName}")
          @@project.save
       end
       
    end


    def getComponentVersion
      manifestpath = Pathname.new('Pods/Manifest.lock')
      @repo_version = Pod::Lockfile.from_file(manifestpath).version(@repoName)
    end

# 检测用户权限

    def checkUserPermission
      transhomepage
      return unless @repohomepage
      raise "没有权限访问 #{@repoName} 源码,源码地址：#{@repohomepage}" unless Pod::HTTP::validate_url(@repohomepage).success?
    end

    def transhomepage
      return if @repohomepage
      @repohomepage = @repo.gsub('git@code.byted.org:','https://code.byted.org/') if @repo.include? 'git@code.byted.org:'
      @repohomepage = @repohomepage.gsub('.git','')
    end

# 获取 pod 的基本信息

    def requirePodspecInfo
      transform_path = Pathname.new('Pods/transform.json')
      if File.exist? transform_path
        repo_infos = JSON.parse(File.read(transform_path))
        if repo_infos.has_key? @repoName
          @checkoutinfo = repo_infos[@repoName]['commit']
          @repo = repo_infos[@repoName]['git']
        end
      end
    end

    def requirePodspecInfoIfNeeded
      return if @repo && @checkoutinfo
      url = "https://ios.bytedance.net/api/v1/repo/all?appType=1&offset=1&limit=10&search=#{@repoName}&appId=-1"
      req = REST.get(url, 'Authorization' => 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJuYW1lIjoid3VzaXpoZW4uYXJjaCIsImlhdCI6MTU0NDYwMjI5MH0.daVOz8_8Er0qTv1c4w5O7nGC9D-doDqlcaXOC4SBo2k')
      pods = JSON.parse(req.body)
      raise "没有在无线研发平台上发现对应的组件：#{@repoName},请重试一下" if pods['data']['count'] == 0
      repo_info = pods['data']['rows'].find { |component| 
        if component['repoName'] == @repoName
          find_version = "http://mobile.bytedance.net/inner_mpaas/v2/repo/searchUpgradeHistoryDetail?repo_id=#{component['id']}&version=#{@repo_version}"
          req = REST.get(find_version)
          info = JSON.parse(req.body)
          if info['err_no'].to_i == 0
            @checkoutinfo = info['data']['commit_id']
            true
          end
        end
        
      }
      raise "没有找到名为 #{@repoName} 的组件" unless repo_info
      raise "没有找到 #{@repoName} #{@repo_version} 版本的 commit 信息，请在 Bus 平台查看" unless repo_info

      @repo = repo_info['repo']
      
      @repohomepage = pods['data']['rows'][0]['repohomepage']
    end

# 获取Git地址

    def establishGitRepo
      cdir = Dir::pwd
      cdir = cdir + '/Pods/' + @repoName 
      Dir.mktmpdir{|dir|
        Pod::CoreUI.print("-- 正在部署 #{@repoName} ")
        Pod::CoreUI.print(" ")
        
        output = `
          cd #{dir}
          git clone -q #{@repo} #{@repoName}
          cd #{@repoName}
          git checkout -q #{@checkoutinfo}
          echo "#{cdir}/.git"
          test -d #{cdir}/.git && rm -rf #{cdir}/.git
          mv -f .git #{cdir} && cd #{cdir} && git reset --hard
        `.lines.to_a
        if $?.exitstatus != 0
          Pod::CoreUI.warn('[ERROR] 部署失败，请产看原因:')
        else
          Pod::CoreUI.warn('部署完毕，请重启 Xcode')
        end
      }
    end

    # 工具方法
    def isInDevPod(pod_name)
      development_pods.find { |group| group.name == pod_name || group.path == pod_name }.class != NilClass
    end

    def isInNormalPods(pod_name)
      pods.find { |group| group.name == pod_name || group.path == pod_name }
    end

    def pod_group(pod_name)
      pod_groups.find { |group| group.name == pod_name || group.path == pod_name }
    end

    def pod_groups
      if development_pods
        return pods + development_pods
      end
      return pods
    end

    def development_pod
      if isContainDevPod
        return @@project.groups.find { |group| group.name == 'Development Pods' }
      end
    end


    def pods
      find = @@project.groups.find { |group| group.name == 'Pods' }
      find.children.objects
    end

    def development_pods
      if isContainDevPod
        return @@project.groups.find { |group| group.name == 'Development Pods' }.children.objects
      end
    end

    def isContainDevPod
      find = @@project.groups.find { |group| group.name == 'Development Pods' }
      find.class != NilClass
    end

  end
end

